==> tecfy_core_package-0.1.36.tar.gz: lib/views/components/entity/many_to_many_with_data.dart <==
part of tecfy_core_client_package;

class ManyToManyWithData extends StatefulWidget {
  final Entity entity;
  final String entityId;
  final String entityMiddle;
  final String propertieMiddle;
  final List<dynamic> items;
  final String label;
  final bool isRequired;
  final void Function(List<dynamic>) onChanged;
  ManyToManyWithData(this.entityId,
      {this.entity,
      this.isRequired,
      this.items,
      this.onChanged,
      this.label,
      this.entityMiddle,
      this.propertieMiddle});
  @override
  _ManyToManyWithDataState createState() => _ManyToManyWithDataState();
}

class _ManyToManyWithDataState extends State<ManyToManyWithData> {
  Entity entity;
  // List<Property> properties;
  List<Map<String, dynamic>> itemList;
  List<dynamic> items;
  bool isLoaded = false;
  @override
  void initState() {
    print('--------------_ManyToManyWithDataState---------------');
    print(widget.entityId);
    if (widget.entity == null) {
      EntityService.entities.keys.forEach((element) {
        print(element);
      });
    }
    print(widget.entity);
    entity = EntityService.getEntity(widget.entityMiddle);
    if (entity == null) {
      print('entity not found !!!      ${widget.entityMiddle}');
      return;
    }
    // properties = entity.properties.where((e)=>e.isEdit).toList();
    // print('entity and props are ok!');
    loadData();
    super.initState();
  }

  loadData() async {
    if (items != null && items.length == 0 && widget.items == null) return;
    items = widget.items ?? [];
    var result = await EntityService().getData(widget.entity);
    if (result.data == null || result.data.length == 0)
      result =
          await EntityService().getData(widget.entity, forceFromServer: true);
    if (!result.success) {
      result.showMessage(context);
      return;
    }
    itemList = [];
    for (Map row in result.data) {
      var otherRows = widget.items
              ?.where((s) => s[widget.propertieMiddle] == row['id'])
              ?.toList() ??
          [];

      // console.log('this.isFilterData',this.isFilterData);
      if (otherRows.length > 0) {
        // itemList.add({'selected' :true, ...otherRows[0],id:row.id,display:row.display});
        var r = otherRows[0];
        r['selected'] = true;
        r['id'] = row['id'];
        r['display'] = row['display'];
        r[widget.propertieMiddle
            .substring(0, widget.propertieMiddle.length - 2)] = {
          'id': row['id'],
          'display': row['display']
        };
        this.itemList.add(r);
      } else {
        row[widget.propertieMiddle] = row['id'];
        row[widget.propertieMiddle
            .substring(0, widget.propertieMiddle.length - 2)] = {
          'id': row['id'],
          'display': row['display']
        };
        row['selected'] = false;
        this.itemList.add(row);
      }
    }
    if (mounted) setState(() {});
  }

  @override
  Widget build(BuildContext context) {
    if (items != widget.items) loadData();
    return itemList == null
        ? CircularProgressIndicator(
            valueColor: AlwaysStoppedAnimation<Color>(ThemeService.mainColor))
        : Column(
            children: itemList
                .map((d) => Column(
                      children: <Widget>[
                        AppCheckBox(
                            initalValue: d['selected'] ?? false,
                            label: d['display'] ?? '!!',
                            onChanged: (v) {
                              d['selected'] = v;
                              setState(() {});
                            }),
                        d['selected'] == true
                            ? InLineForm(widget.entityId,
                                data: d, entity: entity, onChanged: (value) {
                                // save to selected items
                                //print(value);
                                //s[widget.propertieMiddle] == row['id']
                                if (items == null) {
                                  print(
                                      'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX items is null');
                                  print(items);
                                  print(widget.items);
                                  return;
                                }
                                var index = items.indexWhere(
                                    (item) => item['id'] == value['id']);
                                if (index >= 0) {
                                  items.replaceRange(index, index + 1, [value]);
                                } else {
                                  items.add(value);
                                }
                                widget.onChanged(items);
                              })
                            : Container()
                      ],
                    ))
                .toList(),
          );
  }
}

==> tecfy_core_package-0.1.55.tar.gz: lib/views/components/entity/many_to_many_with_data.dart <==
part of tecfy_core_client_package;

class ManyToManyWithData extends StatefulWidget {
  final Entity entity;
  final String entityId;
  final String entityMiddle;
  final String propertieMiddle;
  final List<dynamic> items;
  final String label;
  final bool isRequired;
  final void Function(List<dynamic>) onChanged;
  ManyToManyWithData(this.entityId,
      {this.entity,
      this.isRequired,
      this.items,
      this.onChanged,
      this.label,
      this.entityMiddle,
      this.propertieMiddle});
  @override
  _ManyToManyWithDataState createState() => _ManyToManyWithDataState();
}

class _ManyToManyWithDataState extends State<ManyToManyWithData> {
  Entity entity;
  // List<Property> properties;
  List<Map<String, dynamic>> itemList;
  List<dynamic> items;
  bool isLoaded = false;
  @override
  void initState() {
    print('--------------_ManyToManyWithDataState---------------');
    print(widget.entityId);
    if (widget.entity == null) {
      EntityService.entities.keys.forEach((element) {
        print(element);
      });
    }
    print(widget.entity);
    entity = EntityService.getEntity(widget.entityMiddle);
    if (entity == null) {
      print('entity not found !!!      ${widget.entityMiddle}');
      return;
    }
    // properties = entity.properties.where((e)=>e.isEdit).toList();
    // print('entity and props are ok!');
    loadData();
    super.initState();
  }

  loadData() async {
    if (items != null && items.length == 0 && widget.items == null) return;
    items = widget.items ?? [];
    var result = await EntityService.getData(widget.entity);
    if (result.data == null || result.data.length == 0)
      result = await EntityService.getData(widget.entity);
    if (!result.success) {
      result.showMessage(context);
      return;
    }
    itemList = [];
    for (Map row in result.data) {
      var otherRows = widget.items
              ?.where((s) => s[widget.propertieMiddle] == row['id'])
              ?.toList() ??
          [];

      // console.log('this.isFilterData',this.isFilterData);
      if (otherRows.length > 0) {
        // itemList.add({'selected' :true, ...otherRows[0],id:row.id,display:row.display});
        var r = otherRows[0];
        r['selected'] = true;
        r['id'] = row['id'];
        r['display'] = row['display'];
        r[widget.propertieMiddle
            .substring(0, widget.propertieMiddle.length - 2)] = {
          'id': row['id'],
          'display': row['display']
        };
        this.itemList.add(r);
      } else {
        row[widget.propertieMiddle] = row['id'];
        row[widget.propertieMiddle
            .substring(0, widget.propertieMiddle.length - 2)] = {
          'id': row['id'],
          'display': row['display']
        };
        row['selected'] = false;
        this.itemList.add(row);
      }
    }
    if (mounted) setState(() {});
  }

  @override
  Widget build(BuildContext context) {
    if (items != widget.items) loadData();
    return itemList == null
        ? CircularProgressIndicator(
            valueColor: AlwaysStoppedAnimation<Color>(ThemeService.mainColor))
        : Column(
            children: itemList
                .map((d) => Column(
                      children: <Widget>[
                        AppCheckBox(
                            initalValue: d['selected'] ?? false,
                            label: d['display'] ?? '!!',
                            onChanged: (v) {
                              d['selected'] = v;
                              setState(() {});
                            }),
                        d['selected'] == true
                            ? InLineForm(widget.entityId,
                                data: d, entity: entity, onChanged: (value) {
                                // save to selected items
                                //print(value);
                                //s[widget.propertieMiddle] == row['id']
                                if (items == null) {
                                  print(
                                      'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX items is null');
                                  print(items);
                                  print(widget.items);
                                  return;
                                }
                                var index = items.indexWhere(
                                    (item) => item['id'] == value['id']);
                                if (index >= 0) {
                                  items.replaceRange(index, index + 1, [value]);
                                } else {
                                  items.add(value);
                                }
                                widget.onChanged(items);
                              })
                            : Container()
                      ],
                    ))
                .toList(),
          );
  }
}
